package com.sawerli.plugin;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Base64;
import android.util.Log;

import com.getcapacitor.FileUtils;
import com.getcapacitor.JSObject;
import com.getcapacitor.NativePlugin;
import com.getcapacitor.Plugin;
import com.getcapacitor.PluginCall;
import com.getcapacitor.PluginMethod;
import com.getcapacitor.PluginRequestCodes;


import java.io.ByteArrayOutputStream;
import java.io.IOException;


@NativePlugin(
        requestCodes={CustomCamera.REQUEST_IMAGE_CAPTURE, CustomCamera.REQUEST_IMAGE_PICK, CustomCamera.REQUEST_IMAGE_EDIT}
)
public class CustomCamera extends Plugin {

    // Request codes
    static final int REQUEST_IMAGE_CAPTURE = PluginRequestCodes.CAMERA_IMAGE_CAPTURE;
    static final int REQUEST_IMAGE_PICK = PluginRequestCodes.CAMERA_IMAGE_PICK;
    static final int REQUEST_IMAGE_EDIT = PluginRequestCodes.CAMERA_IMAGE_EDIT;


    // Message constants
    private static final String PERMISSION_DENIED_ERROR = "Unable to access camera, user denied permission request";
    /*private static final String INVALID_RESULT_TYPE_ERROR = "Invalid resultType option";
    private static final String NO_CAMERA_ERROR = "Device doesn't have a camera available";
    private static final String NO_CAMERA_ACTIVITY_ERROR = "Unable to resolve camera activity";
    private static final String IMAGE_FILE_SAVE_ERROR = "Unable to create photo on disk";
    private static final String IMAGE_PROCESS_NO_FILE_ERROR = "Unable to process image, file not found on disk";
    private static final String UNABLE_TO_PROCESS_IMAGE = "Unable to process image";*/

    private String imageFileSavePath;


    @PluginMethod()
    public void getPhoto(PluginCall call) {
        saveCall(call);
        openCamera(call);
    }

    private boolean checkPhotosPermissions(PluginCall call) {
        if(!hasPermission(Manifest.permission.READ_EXTERNAL_STORAGE)) {
            pluginRequestPermission(Manifest.permission.READ_EXTERNAL_STORAGE, REQUEST_IMAGE_CAPTURE);
            return false;
        }
        return true;
    }

    @Override
    protected void handleRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        super.handleRequestPermissionsResult(requestCode, permissions, grantResults);

        Log.d(getLogTag(),"handling request perms result");

        if (getSavedCall() == null) {
            Log.d(getLogTag(),"No stored plugin call for permissions request result");
            return;
        }

        PluginCall savedCall = getSavedCall();

        for (int i = 0; i < grantResults.length; i++) {
            int result = grantResults[i];
            String perm = permissions[i];
            if(result == PackageManager.PERMISSION_DENIED) {
                Log.d(getLogTag(), "User denied camera permission: " + perm);
                savedCall.error(PERMISSION_DENIED_ERROR);
                return;
            }
        }

        if (requestCode == REQUEST_IMAGE_CAPTURE) {
            getPhoto(savedCall);
        }
    }

    private boolean checkCameraPermissions(PluginCall call) {
        // If we want to save to the gallery, we need two permissions
        if(!(hasPermission(Manifest.permission.CAMERA) && hasPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE))) {
            pluginRequestPermissions(new String[] {
                    Manifest.permission.CAMERA,
                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.READ_EXTERNAL_STORAGE
            }, REQUEST_IMAGE_CAPTURE);
            return false;
        }
        // If we don't need to save to the gallery, we can just ask for camera permissions
        if(!hasPermission(Manifest.permission.CAMERA)) {
            pluginRequestPermission(Manifest.permission.CAMERA, REQUEST_IMAGE_CAPTURE);
            return false;
        }
        return true;
    }


    @Override
    protected void handleOnActivityResult(int requestCode, int resultCode, Intent data) {
        super.handleOnActivityResult(requestCode, resultCode, data);

        if (data != null) {
            String action = data.getStringExtra("action");
            if (action != null) {
                if (action.equals(PluginActions.BACK)) {
                    return;
                }
            }

            PluginCall savedCall = getSavedCall();
            //Bundle bundle = data.getBundleExtra("uri");
            String uri = data.getStringExtra("uri");


            if (uri == "" || uri == null) {  // from gallery
                uri = data.getDataString();
            }

            String base64 = "data:image/jpeg;base64,";
            try {
                Bitmap bitmap = MediaStore.Images.Media.getBitmap(this.getActivity().getContentResolver(), Uri.parse(uri));
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                bitmap.compress(Bitmap.CompressFormat.JPEG, 50, baos);
                byte[] bytes = baos.toByteArray();
                base64 = base64 + Base64.encodeToString(bytes, Base64.NO_WRAP);


            } catch (IOException e) {
                e.printStackTrace();
            }


            String webPath = FileUtils.getPortablePath(getContext(), bridge.getLocalUrl(), Uri.parse(uri));
            if (savedCall == null) {
                return;
            }

            //settings = getSettings(savedCall);
            //imageFileSavePath = data.getDataString();
            if (requestCode == REQUEST_IMAGE_CAPTURE) {
                returnResultFromCameraImage(savedCall, Uri.parse(uri), webPath, base64);
            }/* else if (requestCode == REQUEST_IMAGE_PICK) {
            //processPickedImage(savedCall, data);
            }*/
        } else {
            JSObject ret = new JSObject();
            ret.put("action", PluginActions.BACK);
            notifyListeners("backEvent", ret);
       }
    }

    public void returnResultFromCameraImage(PluginCall call, Uri uri, String webPath, String base64) {
        JSObject data = new JSObject();
        data.put("uri", uri.toString());
        data.put("format", "jpeg");
        data.put("webPath", webPath);
        data.put("base64", base64);
        call.resolve(data);
    }


    @Override
    protected Bundle saveInstanceState() {
        Bundle bundle = super.saveInstanceState();
        bundle.putString("cameraImageFileSavePath", imageFileSavePath);
        return bundle;
    }

    @Override
    protected void restoreState(Bundle state) {
        String storedImageFileSavePath = state.getString("cameraImageFileSavePath");
        if (storedImageFileSavePath != null) {
            imageFileSavePath = storedImageFileSavePath;
        }
    }

    public void openCamera(final PluginCall call) {
        if (checkCameraPermissions(call) && checkPhotosPermissions(call)) {

            Intent customCamActivityIntent = new Intent(this.getActivity(), CustomCameraActivity.class);
            startActivityForResult(call, customCamActivityIntent, REQUEST_IMAGE_CAPTURE);
        }
    }


    @PluginMethod()
    public void echo(PluginCall call) {
        String value = call.getString("value");

        JSObject ret = new JSObject();
        ret.put("value", value);
        call.success(ret);
    }
}
